import asyncio
import os

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "myauth.settings.local")
# shakes head in shame... this import must be under that ^
from aadiscordbot import launcher  # nopep8


def main():
    loop = asyncio.get_event_loop()
    launcher.run_bot()


if __name__ == "__main__":
    main()



import asyncio
import logging

from discord import AllowedMentions
from discord.ext import commands

from django.conf import settings

logger = logging.getLogger(__name__)


class Remind(commands.Cog):
    """
    Set reminders
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.slash_command(name='remind', description="Set a Reminder", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def reminder(self, ctx, reminder: str, seconds: int = 0, minutes: int = 0, hours: int = 0, days: int = 0):
        counter = seconds + minutes * 60 + hours * 60 * 60 + days * 60 * 60 * 24
        await ctx.respond(f"Alright, I will remind you about {reminder} in {seconds}s {minutes}m {hours}h {days}d.", allowed_mentions=AllowedMentions(everyone=False, roles=False, users=False))
        await asyncio.sleep(counter)
        await ctx.respond(f"{ctx.user.mention} Hi, you asked me to remind you about {reminder}, {seconds}s {minutes}m {hours}h {days}d ago.", allowed_mentions=AllowedMentions(everyone=False, roles=False, users=[ctx.user]))


def setup(bot):
    bot.add_cog(Remind(bot))



import logging

from discord.colour import Color
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands

from django.conf import settings

from aadiscordbot import __branch__, __version__
from aadiscordbot.app_settings import get_site_url

logger = logging.getLogger(__name__)


class AbuseTwo(commands.Cog):
    """
    Emojis for abusing reasons...!
    """

    def __init__(self, bot):
        self.bot = bot

    # Ariel is wrong, always.
    @commands.Cog.listener("on_message")
    async def respond_to_abusee(self, message):
        if message.author.id == settings.DISCORD_BOT_ABUSEE_ID:
            await message.add_reaction(getattr(settings, 'DISCORD_BOT_ABUSEE_EMOJI', '👎'))


def setup(bot):
    bot.add_cog(AbuseTwo(bot))



# Generated by Django 4.0.6 on 2022-10-22 02:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0008_channels_deleted'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='quotemessage',
            options={'default_permissions': (), 'permissions': (('quote_save', 'Can save quotes'),),
                     'verbose_name': 'Quote Message', 'verbose_name_plural': 'Quote Messages'},
        ),
    ]



import datetime
import logging

import pendulum
from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands

from django.conf import settings
from django.utils import timezone

from allianceauth.timerboard.models import Timer

from aadiscordbot.app_settings import timerboard_active
from aadiscordbot.cogs.utils.decorators import message_in_channels

logger = logging.getLogger(__name__)


class Timers(commands.Cog):
    """
    TimerBoard Stuffs!
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    @message_in_channels(settings.ADMIN_DISCORD_BOT_CHANNELS)
    async def timer(self, ctx):
        """
        Gets the Next Timer!
        :param ctx:
        :return:
        """
        next_timer = Timer.objects.filter(corp_timer=False,
                                          eve_time__gte=datetime.datetime.utcnow().replace(tzinfo=timezone.utc)).first()
        time_until = pendulum.now(tz="UTC").diff_for_humans(
            next_timer.eve_time, absolute=True
        )
        embed = Embed(title="Next Timer")
        embed.description = next_timer.details
        if next_timer.objective == "Friendly":
            embed.colour = Color.blue()
        elif next_timer.objective == "Hostile":
            embed.colour = Color.red()
        else:
            embed.colour = Color.white()
        try:
            embed.set_footer(text="Added By {}".format(
                next_timer.eve_character.character_name))
        except Exception as e:
            logger.error(e)
            pass

        embed.add_field(
            name="Structure:",
            value=next_timer.structure
        )
        embed.add_field(
            name="Location:",
            value=f"{next_timer.system} - {next_timer.planet_moon}"
        )
        embed.add_field(
            name="Eve Time:",
            value=next_timer.eve_time.strftime("%Y-%m-%d %H:%M"),
            inline=False
        )
        return await ctx.send(embed=embed)


def setup(bot):
    if timerboard_active():
        bot.add_cog(Timers(bot))
    else:
        logger.debug("Timerboard not installed")



# Generated by Django 4.0.6 on 2022-09-19 04:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0007_quotemessage'),
    ]

    operations = [
        migrations.AddField(
            model_name='channels',
            name='deleted',
            field=models.BooleanField(default=False, verbose_name='Deleted'),
        ),
    ]



import json
import logging

from aaprom.collectors.discordbot import bot_slash_command, bot_tasks_executed
from discord.ext import commands

from ..app_settings import (
    DISCORD_BOT_FAILURE_MESSAGES_CHANNEL, DISCORD_BOT_SEND_FAILURE_MESSAGES,
)
from ..tasks import send_message

logger = logging.getLogger(__name__)


class PromExporter(commands.Cog):

    def __init__(self, bot):
        self.bot = bot

    @commands.Cog.listener("on_application_command")
    async def internal_on_application_command(self, ctx):
        bot_slash_command.labels(command=str(ctx.command), user=str(
            ctx.author), guild=str(ctx.guild)).inc()

    @commands.Cog.listener("on_authbot_task_completed")
    async def on_authbot_task_completed(self, task):
        bot_tasks_executed.labels(task=str(task), state="success").inc()

    @commands.Cog.listener("on_authbot_task_failed")
    async def on_authbot_task_failed(self, task, args, kwargs, error):
        bot_tasks_executed.labels(task=str(task), state="failed").inc()
        if DISCORD_BOT_SEND_FAILURE_MESSAGES and DISCORD_BOT_FAILURE_MESSAGES_CHANNEL:
            message = [f"Bot Task Failed <{error}>"]
            message.append(f"ARGS: ```\n{json.dumps(args)[0:1000]}\n```")
            message.append(f"KWARGS: ```\n{json.dumps(kwargs)[0:1000]}\n```")
            send_message(message="\n".join(message),
                         channel_id=DISCORD_BOT_FAILURE_MESSAGES_CHANNEL)


def setup(bot):
    bot.add_cog(PromExporter(bot))



import logging

from discord import Embed, User, option
from discord.ext import commands

from django.conf import settings

logger = logging.getLogger(__name__)


class FactionWar(commands.Cog):
    """
    FAction Warfare related helpfull stuffs... Cause moar blues = moar drama....
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.slash_command(name='facwar_km', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    @option("esi_link", description="External ESI Killmail link from in game!")
    async def facwar_km(self, ctx, esi_link: str):
        """
            Check an esi killmail link and confirm members were in faction warfare at the time.
        """
        await ctx.defer()
        if not esi_link.startswith("https://esi.evetech.net/"):
            return await ctx.respond(f"not a valid ESI Killmail link, please try again...")
        km_json = {}
        async with self.bot.session.get(esi_link) as km:
            km_json = await km.json()

        names_to_fetch = set()
        names_to_fetch.add(km_json['victim']['character_id'])
        names_to_fetch.add(km_json['solar_system_id'])
        names_to_fetch.add(km_json['victim']['ship_type_id'])
        names_to_fetch.add(km_json['victim']['character_id'])
        vic_fac = km_json['victim'].get('faction_id', False)
        if vic_fac:
            names_to_fetch.add(km_json['victim'].get('faction_id'))
        for a in km_json['attackers']:
            if a.get('character_id', False):
                names_to_fetch.add(a['character_id'])
                a_fac = a.get('faction_id', False)
                if a_fac:
                    names_to_fetch.add(a.get('faction_id'))
        names = {}
        names_url = f"https://esi.evetech.net/latest/universe/names/?datasource=tranquility"
        names_payload = list(names_to_fetch)
        async with self.bot.session.post(names_url, json=names_payload) as nm:
            nms = await nm.json()
            for n in nms:
                names[n['id']] = n['name']

        detail_needed = False or not vic_fac
        e = Embed(title="Kill Report")
        fac_attackers = []
        non_fac_attackers = []
        messages = []

        if vic_fac:
            messages.append(
                f"`{names[km_json['victim']['character_id']]}` was in Faction `{names[km_json['victim'].get('faction_id')]}` when they lost their `{names[km_json['victim']['ship_type_id']]}` in `{names[km_json['solar_system_id']]}`")
        else:
            messages.append(
                f"`{names[km_json['victim']['character_id']]}` was **NOT** in Faction Warfare when they lost their `{names[km_json['victim']['ship_type_id']]}` in `{names[km_json['solar_system_id']]}`")

        for a in km_json['attackers']:
            if a.get('character_id', False):
                a_fac = a.get('faction_id', False)
                if a_fac:
                    fac_attackers.append(
                        f"{names[a['character_id']]} ({names[a['faction_id']]})")
                else:
                    non_fac_attackers.append(f"{names[a['character_id']]}")
                    detail_needed = True

        if len(fac_attackers):
            fac_attack_msg = f"\n".join(fac_attackers)
            messages.append("\n**Faction Warfare Attackers**")
            messages.append(f"```\n{fac_attack_msg}\n```")

        if len(non_fac_attackers):
            non_fac_attackers_msg = f"\n".join(non_fac_attackers)
            messages.append("\n**Non-Faction Warfare Attackers**")
            messages.append(f"```\n{non_fac_attackers_msg}\n```")

        e.description = "\n".join(messages)

        if detail_needed:
            e.color = 0xe74c3c
        else:
            e.color = 0x2ecc71

        return await ctx.respond(embed=e)


def setup(bot):
    bot.add_cog(FactionWar(bot))



"""
"Time" cog for discordbot - https://github.com/pvyParts/allianceauth-discordbot
"""

import logging
from datetime import datetime

from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands

from django.conf import settings

logger = logging.getLogger(__name__)


class Time(commands.Cog):
    """
    A series of Time tools
    """

    def __init__(self, bot):
        self.bot = bot

    def build_embed(self):
        fmt_utc = "%H:%M:%S (UTC)\n%A %d. %b %Y"

        embed = Embed(title="Time")
        embed.colour = Color.green()

        embed.add_field(
            name="EVE Time",
            value=datetime.utcnow().strftime(fmt_utc),
            inline=False,
        )

        return embed

    @commands.command(pass_context=True)
    async def time(self, ctx):
        """
        Returns EVE Time
        """

        return await ctx.send(embed=self.build_embed())

    @commands.slash_command(name="time", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def time_slash(self, ctx):
        """
        Returns EVE Time
        """

        return await ctx.respond(embed=self.build_embed())


def setup(bot):
    """
    setup the cog
    :param bot:
    """

    # Only load if there is no other Time cog loaded already
    # aa-timezones>=1.10.0 for example
    if bot.get_cog("Time") is None:
        bot.add_cog(Time(bot))



from typing import Any, List

from discord.ext.commands.errors import CheckFailure

from aadiscordbot.app_settings import get_site_url


class NotAuthenticated(CheckFailure):
    """
    Custom Exception to handle any users not Authenticated with AA
    """

    def __init__(self, *args: Any) -> None:

        message = f"You must be an Authenticated user to run this command {get_site_url()}"
        super().__init__(message, *args)



import logging

from discord import reaction
from discord.ext import commands
from discord.utils import get

from allianceauth.services.modules.discord.models import DiscordUser

from aadiscordbot.cogs.utils.decorators import sender_has_perm

from ..models import ReactionRoleBinding, ReactionRoleMessage

logger = logging.getLogger(__name__)


class Reactions(commands.Cog):
    """
    Auth Roles as reaction roles. With optional Public Access at a message level.
    """

    def __init__(self, bot):
        self.bot = bot

    async def clean_emojis(self, payload):
        gld = get(self.bot.guilds, id=payload.guild_id)
        chan = gld.get_channel(payload.channel_id)
        msg = await chan.fetch_message(payload.message_id)
        rr_binds = ReactionRoleBinding.objects.filter(
            message=payload.message_id).values_list('emoji_text', flat=True)
        for e in msg.reactions:
            if isinstance(e.emoji, str):
                em = e.emoji.encode('utf-8')
                es = e.emoji
                if str(em) not in rr_binds and es not in rr_binds:
                    async for u in e.users():
                        await e.remove(u)
            else:
                em = e.emoji.name.encode('utf-8')
                es = e.emoji.name
                if str(em) not in rr_binds and es not in rr_binds:
                    async for u in e.users():
                        await e.remove(u)

    @commands.command(pass_context=True)
    @sender_has_perm('aadiscordbot.manage_reactions')
    async def rr(self, ctx):
        ReactionRoleMessage.objects.create(
            message=ctx.message.id, name=f"{ctx.message.channel.name} RR Message {ctx.message.id}")
        return await ctx.message.add_reaction("👍")

    @commands.Cog.listener("on_raw_reaction_add")
    async def add_react_listener(self, payload):
        """
            add role or group
        """
        if payload.user_id == self.bot.user.id:
            return True
        try:
            rr_msg = ReactionRoleMessage.objects.get(
                message=payload.message_id)
            # do we have a binding?
            emoji = payload.emoji.name.encode('utf-8')
            if payload.emoji.id is not None:
                emoji = payload.emoji.id
            try:
                rr_binds = ReactionRoleBinding.objects.get(
                    message=rr_msg, emoji=emoji)
            except ReactionRoleBinding.DoesNotExist:
                try:
                    rr_binds = ReactionRoleBinding.objects.get(
                        message=rr_msg, emoji=payload.emoji.name)
                except:
                    # admin adding new role?
                    if DiscordUser.objects.get(uid=payload.user_id).user.has_perm("aadiscordbot.manage_reactions"):
                        gld = get(self.bot.guilds, id=payload.guild_id)
                        chan = gld.get_channel(payload.channel_id)
                        msg = await chan.fetch_message(payload.message_id)
                        ReactionRoleBinding.objects.create(
                            message=rr_msg, emoji=emoji, emoji_text=payload.emoji.name.encode('utf-8'))
                        await msg.add_reaction(payload.emoji)
                        return await self.clean_emojis(payload)
            if rr_binds.group:
                try:
                    user = DiscordUser.objects.get(uid=payload.user_id)
                    user = user.user
                    if rr_binds.group not in user.groups.all():
                        user.groups.add(rr_binds.group)
                except DiscordUser.DoesNotExist:
                    if rr_msg.non_auth_users:
                        try:
                            gld = get(self.bot.guilds, id=payload.guild_id)
                            role = get(gld.roles, name=rr_binds.group.name)
                            user = get(gld.members, id=payload.user_id)
                            await user.add_roles(role)
                        except AttributeError:
                            pass  # No group or user or guild. zero fks given
                    return await self.clean_emojis(payload)
            await self.clean_emojis(payload)
        except ReactionRoleMessage.DoesNotExist:
            pass

    @commands.Cog.listener("on_raw_reaction_remove")
    async def rem_react_listener(self, payload):
        """
            rem role or group
        """
        if payload.user_id == self.bot.client_id:
            return True
        try:
            rr_msg = ReactionRoleMessage.objects.get(
                message=payload.message_id)
            # do we have a binding?
            emoji = payload.emoji.name.encode('utf-8')
            if payload.emoji.id is not None:
                emoji = payload.emoji.id
            try:
                rr_binds = ReactionRoleBinding.objects.get(
                    message=rr_msg, emoji=emoji)
            except ReactionRoleBinding.DoesNotExist:
                try:
                    rr_binds = ReactionRoleBinding.objects.get(
                        message=rr_msg, emoji=payload.emoji.name)
                except ReactionRoleBinding.DoesNotExist:
                    return await self.clean_emojis(payload)
            try:
                user = DiscordUser.objects.get(uid=payload.user_id)
                user = user.user
                if rr_binds.group:
                    if rr_binds.group in user.groups.all():
                        user.groups.remove(rr_binds.group)
            except DiscordUser.DoesNotExist:
                if rr_msg.non_auth_users:
                    try:
                        gld = get(self.bot.guilds, id=payload.guild_id)
                        role = get(gld.roles, name=rr_binds.group.name)
                        user = get(gld.members, id=payload.user_id)
                        await user.remove_roles(role)
                    except AttributeError:
                        pass  # No group or user or guild. zero fks given
                return await self.clean_emojis(payload)
            await self.clean_emojis(payload)
        except ReactionRoleMessage.DoesNotExist:
            pass


def setup(bot):
    bot.add_cog(Reactions(bot))



from allianceauth import hooks
from allianceauth.authentication.models import State, UserProfile
from allianceauth.services.hooks import MenuItemHook, UrlHook

from . import app_settings, urls

"""
class CeleryMenu(MenuItemHook):
    def __init__(self):
        MenuItemHook.__init__(self, 'Celery Tasks',
                              'fa fa-clock fa-fw',
                              'celeryanalytics:show_tasks',
                              navactive=['celeryanalytics:show_tasks'])

    def render(self, request):
        if request.user.is_staff:
            return MenuItemHook.render(self, request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return CeleryMenu()

@hooks.register('url_hook')
def register_url():
    return UrlHook(urls, 'celeryanalytics', r'^celeryanalytics/')
"""


@hooks.register('discord_cogs_hook')
def register_cogs():
    return app_settings.DISCORD_BOT_COGS



from solo.models import SingletonModel

from django.conf import settings
from django.contrib.auth.models import Group
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import gettext_lazy as _

from allianceauth.services.modules.discord.models import DiscordUser


class DiscordBot(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ('basic_access', 'Can access this app.'),
            ('member_command_access', 'can access the member commands.')
        )


class Servers(models.Model):
    """Servers and their ID"""

    server = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=100)

    class Meta:
        verbose_name = 'Server'
        verbose_name_plural = 'Servers'

    def __str__(self):
        return f'{self.name}'


class Channels(models.Model):
    """Channel IDs, Names and the Server they belong to"""

    server = models.ForeignKey(Servers, on_delete=models.CASCADE)
    channel = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=100)
    deleted = models.BooleanField(_("Deleted"), default=False)

    def __str__(self):
        return f'"{self.name}" On "{self.server.name}"'

    class Meta:
        verbose_name = 'Channel'
        verbose_name_plural = 'Channels'


class ReactionRoleMessage(models.Model):
    """Reaction Role Handler"""

    message = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=100)

    non_auth_users = models.BooleanField(
        default=False, help_text="Can Non Authed/public discord members gain groups from this Reaction Roles Message")

    def __str__(self):
        return f'{self.name}'

    class Meta:
        permissions = (
            ('manage_reactions', 'Can Manage Reaction Roles'),
        )


class ReactionRoleBinding(models.Model):
    """Reaction Links"""

    group = models.ForeignKey(
        Group, on_delete=models.CASCADE, null=True, default=None)
    emoji = models.CharField(max_length=100)
    emoji_text = models.CharField(max_length=100)
    message = models.ForeignKey(ReactionRoleMessage, on_delete=models.CASCADE)

    def __str__(self):
        try:
            b = eval(self.emoji_text).decode('utf-8')
        except:
            b = self.emoji_text
        return f'{self.message.name}: {b} ({self.group})'


class AuthBotConfiguration(models.Model):

    admin_users = models.ManyToManyField(DiscordUser, blank=True)

    def __str__(self):
        return "AuthBot Configuration"

    def save(self, *args, **kwargs):
        if not self.pk and AuthBotConfiguration.objects.exists():
            # Force a single object
            raise ValidationError(
                'Only one Settings Model can there be at a time! No Sith Lords there are here!')
        self.pk = self.id = 1  # If this happens to be deleted and recreated, force it to be 1
        return super().save(*args, **kwargs)


class WelcomeMessage(models.Model):
    message = models.TextField(_("Welcome Message"))
    authenticated = models.BooleanField(_("Valid for Authenticated Users"))
    unauthenticated = models.BooleanField(
        _("Valid for Un-Authenticated Users"))

    class Meta:
        default_permissions = ()
        verbose_name = 'Welcome Message'
        verbose_name_plural = 'Welcome Messages'


class GoodbyeMessage(models.Model):
    message = models.TextField(_("Goodbye Message"))
    authenticated = models.BooleanField(_("Valid for Authenticated Users"))
    unauthenticated = models.BooleanField(
        _("Valid for Un-Authenticated Users"))

    class Meta:
        default_permissions = ()
        verbose_name = 'Goodbye Message'
        verbose_name_plural = 'Goodbye Messages'


class QuoteMessage(models.Model):
    """A saved discord message, Used by the Quote cog"""
    server = models.ForeignKey(Servers, on_delete=models.CASCADE)
    channel = models.ForeignKey(Channels, on_delete=models.CASCADE)
    message = models.BigIntegerField(primary_key=True)
    content = models.CharField(max_length=1000)
    datetime = models.DateTimeField(blank=True, null=True)
    author = models.PositiveBigIntegerField()
    author_nick = models.CharField(max_length=50, blank=True, null=True)
    reference = models.CharField(
        max_length=100, help_text="Nickname for this quote")

    class Meta:
        default_permissions = ()
        verbose_name = 'Quote Message'
        verbose_name_plural = 'Quote Messages'
        permissions = (
            ('quote_save', 'Can save quotes'),
        )


class TicketGroups(SingletonModel):
    groups = models.ManyToManyField(
        Group, blank=True, help_text="Pingable groups for ticketing")
    ticket_channel = models.ForeignKey(
        Channels, on_delete=models.SET_NULL, null=True, default=None)

    class Meta:
        default_permissions = ()
        verbose_name = 'Ticket Cog Configuration'
        verbose_name_plural = 'Ticket Cog Configuration'

    def __str__(self):
        return "Ticket Cog Configuration"



from discord.ext import commands


class Context(commands.Context):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    @property
    def session(self):
        return self.bot.session



from django.apps import AppConfig

from . import __version__


class AADiscordBotConfig(AppConfig):
    name = 'aadiscordbot'
    label = 'aadiscordbot'
    verbose_name = f'Discord Bot v{__version__}'

    # def ready(self):
    #    import aadiscordbot.signals



# Cog Stuff
import logging
from typing import Optional

import discord
from discord import ChannelType, Embed, Message, command, ui
from discord.ext import commands

# AA Contexts
from django.conf import settings
from django.utils import timezone

from .. import models

logger = logging.getLogger(__name__)


def get_groups():
    groups = models.TicketGroups.get_solo().groups.all().order_by('name')
    out = []
    for g in groups:
        out.append(
            discord.SelectOption(
                label=f"{g.name}"
            )
        )
    return out


class TicketDropdown(discord.ui.Select):
    """
        Group Dropdown for discord message to summon a private help channel.
    """

    def __init__(self):
        super().__init__(
            placeholder="Who do you need help from?",
            options=get_groups(),
        )

    async def callback(self, interaction: discord.Interaction):
        sup_channel = models.TicketGroups.get_solo().ticket_channel.channel
        ch = interaction.guild.get_channel(sup_channel)
        grp = discord.utils.get(interaction.guild.roles, name=self.values[0])
        th = await ch.create_thread(name=f"{interaction.user.display_name} | {self.values[0]} | {timezone.now().strftime('%Y-%m-%d %H:%M')}",
                                    auto_archive_duration=10080,
                                    type=discord.ChannelType.private_thread,
                                    reason=None)
        msg = f"<@{interaction.user.id}> needs help!, Someone from <@&{grp.id}> will get in touch soon!"
        embd = Embed(title="Private Thread Guide",
                     description="To add a person to this thread simply `@ping` them. This works with `@groups` as well to bulk add people to the channel. Use wisely, abuse will not be tolerated.\n\nThis is a beta feature if you experience issues please contact the admins. :heart:")
        await th.send(msg, embed=embd)
        await interaction.response.edit_message(content="Ping in the thread created for urgent help!", view=None)


class HelpView(ui.View):
    """
        View for picking a group to assign a help thread too
    """

    def __init__(self):
        super().__init__(TicketDropdown())


class HelpCog(commands.Cog):
    """
        Help Ticket Cog Things
    """

    def __init__(self, bot):
        self.bot = bot

    @command(name='help', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def slash_halp(
        self,
        ctx,
    ):
        """
            Help me authbot i dont know who to ping!
        """
        await ctx.defer(ephemeral=True)
        return await ctx.respond(view=HelpView())

    @command(name='close_ticket', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def slash_close(
        self,
        ctx,
    ):
        """
            Mark help thread as completed and archive it!
        """
        ch = ctx.channel
        if ch.type != ChannelType.private_thread:
            return await ctx.respond("Not a private thread!", ephemeral=True)
        await ctx.defer()
        embd = Embed(title="Thread Marked Complete",
                     description=f"{ctx.user.display_name} has marked this thread as completed. To reopen simply start chatting again.")
        await ctx.respond(embed=embd)
        return await ch.archive()

    @commands.message_command(name="Create Help Ticket", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def reverse_halp(self, ctx, message: Message):
        sup_channel = models.TicketGroups.get_solo().ticket_channel.channel
        ch = message.guild.get_channel(sup_channel)
        files = []
        for a in message.attachments:
            files.append(a.proxy_url)
        _f = "\n".join(files)

        th = await ch.create_thread(name=f"{message.author.display_name} | {message.id} | {timezone.now().strftime('%Y-%m-%d %H:%M')}",
                                    #message=f"Ping in here if your request is urgent!",
                                    auto_archive_duration=10080,
                                    type=discord.ChannelType.private_thread,
                                    reason=None)
        msg = f"hi, <@{message.author.id}>, <@{ctx.author.id}> wants clarification on this message\n\n```{message.content}```\n\n{message.jump_url}\n{_f}"
        embd = Embed(title="Private Thread Guide",
                     description="To add a person to this thread simply `@ping` them. This works with `@groups` as well to bulk add people to the channel. Use wisely, abuse will not be tolerated.")

        await th.send(msg, embed=embd)


def setup(bot):
    bot.add_cog(HelpCog(bot))



# Generated by Django 3.2.8 on 2022-01-26 05:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('discord', '0003_big_overhaul'),
        ('aadiscordbot', '0004_settings'),
    ]

    operations = [
        migrations.AddField(
            model_name='reactionrolemessage',
            name='non_auth_users',
            field=models.BooleanField(
                default=False, help_text='Can Non Authed/public discord members gain groups from this Reaction Roles Message'),
        ),
        migrations.AlterField(
            model_name='authbotconfiguration',
            name='admin_users',
            field=models.ManyToManyField(blank=True, to='discord.DiscordUser'),
        ),
    ]



import logging
import os

from discord.ext import commands

from django.core.exceptions import ObjectDoesNotExist

from allianceauth.services.modules.discord.models import DiscordUser

from aadiscordbot.app_settings import get_admins
from aadiscordbot.cogs.utils.exceptions import NotAuthenticated

logger = logging.getLogger(__name__)

# i dont want to do this, but the below object get wont work without it, investigate.
os.environ["DJANGO_ALLOW_ASYNC_UNSAFE"] = "true"


def has_perm(id, perm: str):
    if id in get_admins():
        return True
    try:
        user = DiscordUser.objects.get(uid=id)
        has_perm = user.user.has_perm(perm)

        if has_perm:
            return True
        else:
            raise commands.MissingPermissions([perm])
    except DiscordUser.DoesNotExist as e:
        logger.error(e)
        raise NotAuthenticated


def sender_has_perm(perm: str):
    """
    Permission Decorator: Does the user have x Django Permission
    """
    def predicate(ctx):
        if hasattr(ctx, "user"):
            return has_perm(ctx.user.id, perm)
        else:
            return has_perm(ctx.author.id, perm)  # !Commands

    return commands.check(predicate)


def has_all_perms(id, perms: list):
    if id in get_admins():
        return True
    try:
        has_perm = DiscordUser.objects.get(uid=id).user.has_perms(perms)
        if has_perm:
            return True
        else:
            raise commands.MissingPermissions(perms)
    except Exception as e:
        logger.error(e)
        raise NotAuthenticated


def sender_has_all_perms(perms: list):
    """
    Permission Decorator: Does the user have x and y Django Permission
    """
    def predicate(ctx):
        if hasattr(ctx, "user"):
            return has_all_perms(ctx.user.id, perms)
        else:
            return has_all_perms(ctx.author.id, perms)  # !Commands

    return commands.check(predicate)


def has_any_perm(id, perms: list):
    if id in get_admins():
        return True
    for perm in perms:
        try:
            has_perm = DiscordUser.objects.get(uid=id).user.has_perm(perm)
            if has_perm:
                return True
        except Exception as e:
            logger.error(e)
            raise NotAuthenticated
    raise commands.MissingPermissions(["One of the following: "] + perms)


def sender_has_any_perm(perms: list):
    """
    Permission Decorator: Does the user have x or y Django Permission
    """
    def predicate(ctx):
        if hasattr(ctx, "user"):
            return has_any_perm(ctx.user.id, perms)
        else:
            return has_any_perm(ctx.author.id, perms)  # !Commands

    return commands.check(predicate)


def is_admin(id):
    if id in get_admins():
        return True
    else:
        raise commands.CheckFailure("You are not an Administrator")


def sender_is_admin():
    """
    Permission Decorator: is the User configured as AuthBotConfiguration.objects.get(pk=1).admin_users
    """
    def predicate(ctx):
        if hasattr(ctx, "user"):
            return is_admin(ctx.user.id)
        else:
            return is_admin(ctx.author.id)  # !Commands
    return commands.check(predicate)


def in_channels(channel, channels):
    if channel in channels:
        return True
    else:
        raise commands.CheckFailure("Not an Allowed Channel for this Command")


def message_in_channels(channels: list):
    def predicate(ctx):
        return in_channels(ctx.message.channel.id, channels)
    return commands.check(predicate)


def is_authenticated(id):
    try:
        DiscordUser.objects.get(uid=id)
        return True
    except ObjectDoesNotExist:
        raise NotAuthenticated


def sender_is_authenticated():
    """
    Permission Decorator: Is the user Authenticated
    """
    def predicate(ctx):
        return is_authenticated(ctx.user.id)
    return commands.check(predicate)



from esi.clients import esi_client_factory


class EsiResponseClient:
    def __init__(self):
        self._client = None

    @property
    def client(self):
        if self._client is None:
            self._client = esi_client_factory()  # all groups latest
        return self._client


esi = EsiResponseClient()



import logging

from discord import AllowedMentions, Interaction, NotFound
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands
from discord.ui import InputText, Modal

from django.conf import settings
from django.db import IntegrityError

from aadiscordbot.cogs.utils.decorators import sender_has_perm
from aadiscordbot.models import Channels, QuoteMessage, Servers

logger = logging.getLogger(__name__)


class Quote(commands.Cog):
    """
    Save and recall discord messages for quick reference (memes)
    """

    def __init__(self, bot):
        self.bot = bot

    quote_commands = SlashCommandGroup("quote", "Save and Recall Quotes (memes)", guild_ids=[
                                       int(settings.DISCORD_GUILD_ID)])

    class QuoteReference(Modal):
        def __init__(self):
            super().__init__(title="Save Quote")

            self.quote_reference = None

            self.add_item(
                InputText(
                    label="Reference",
                    placeholder="A reference to recall this Quote later, must be unique"
                )
            )

        async def callback(self, interaction: Interaction):
            self.quote_reference = self.children[0].value
            await interaction.response.send_message(f"Attempting to save Quote with reference: {self.quote_reference}", ephemeral=True)
            self.stop()

    @commands.message_command(name="Save Quote", description="Save this message as a Quote", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    @sender_has_perm("aadiscordbot.quote_save")
    async def savequote(self, ctx, message):
        """
        Save a discord message to a Django model for reference later
        """
        ask_for_reference_text = Quote.QuoteReference()
        await ctx.send_modal(ask_for_reference_text)
        await ask_for_reference_text.wait()

        if message.author.nick is None:
            nickname_excepted = message.author.name
        else:
            nickname_excepted = message.author.nick

        try:
            QuoteMessage.objects.update_or_create(
                server_id=message.guild.id,
                channel_id=message.channel.id,
                message=message.id,
                content=message.content,
                datetime=message.created_at,
                author=message.author.id,
                author_nick=(nickname_excepted),
                reference=ask_for_reference_text._children[0].value
            )
            return await message.reply(f"Saved Quote as {ask_for_reference_text._children[0].value}", allowed_mentions=AllowedMentions(replied_user=False))
        except IntegrityError:
            return await ctx.respond(f"Reference is not Unique, Please try again", ephemeral=True)
        except Exception as e:
            return await ctx.respond(e)

    @quote_commands.command(name="recall", description="Recall a Quote from its reference", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    @sender_has_perm("aadiscordbot.quote_recall")
    async def quote_recall(self, ctx, reference: str):
        quote = QuoteMessage.objects.get(reference=reference)
        try:
            await ctx.respond(
                embed=await reconstruct_message(
                    self,
                    ctx,
                    message_id=quote.message
                )
            )
        except Exception as e:
            logger.error(e)

    @commands.user_command(name="View Quotes", description="View a users saved Quotes", guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def recall_user_quotes(self, ctx, user):
        try:
            quotes = QuoteMessage.objects.filter(author=user.id)
        except Exception as e:
            logger.error(e)
        embed = Embed(title=user.name)
        for quote in quotes:
            embed.add_field(
                name=quote.reference,
                value=quote.content,
            )

        return await ctx.respond(embed=embed, ephemeral=True)


async def reconstruct_message(self, ctx, message_id) -> Embed:
    quote = QuoteMessage.objects.get(message=message_id)

    discord_user = ctx.guild.get_member(quote.author)
    if discord_user is None:
        ctx.guild.fetch_member(quote.author)

    discord_channel = ctx.guild.get_channel(quote.channel_id)
    if discord_channel is None:
        try:
            discord_channel = ctx.guild.fetch_channel(quote.channel_id)
            discord_channel_text = discord_channel.mention
        except Exception:
            discord_channel_text = quote.channel.name
    else:
        discord_channel_text = discord_channel.mention

    discord_message = await discord_channel.fetch_message(message_id)

    embed = Embed(title=quote.reference)
    if discord_user is not str:
        embed.set_author(name=discord_user.nick,
                         icon_url=discord_user.avatar.url)
    embed.description = f"> {quote.content}\n\n{discord_channel_text}"
    embed.timestamp = quote.datetime
    if discord_message is not None:
        embed.url = discord_message.jump_url

    return embed


def setup(bot):
    bot.add_cog(Quote(bot))



import logging
from random import randrange

from discord.ext import commands

from django.conf import settings

from aadiscordbot import __branch__, __version__

logger = logging.getLogger(__name__)


class EightBall(commands.Cog):
    """
    All about me!
    """

    def __init__(self, bot):
        self.bot = bot

    def eightball(self):
        replies = [
            "It is certain",
            "It is decidedly so",
            "Without a doubt",
            "Yes definitely!",
            "As I see it, yes",
            "Most likely",
            "Outlook good",
            "Yes",
            "Signs point to yes",
            "Reply hazy, try again",
            "Better not tell you now",
            "Cannot predict now",
            "Concentrate and ask again",
            "Don't count on it",
            "My reply is no",
            "My sources say no",
            "Outlook not so good",
            "Very doubtful",
        ]
        return replies[randrange(0, len(replies)-1)]

    @commands.command(pass_context=True, aliases=['8ball'])
    async def meb(self, message):
        """
        8 ball go brrrr
        """
        await message.reply(self.eightball())

    @commands.slash_command(name='8ball', guild_ids=[int(settings.DISCORD_GUILD_ID)])
    async def meb_slash(self, ctx, question: str):
        await ctx.respond(f" You Asked: `{question}`\n\n{self.eightball()}")


def setup(bot):
    bot.add_cog(EightBall(bot))



# Generated by Django 4.0.6 on 2022-09-17 05:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aadiscordbot', '0005_rr_fixes_blank_admins'),
    ]

    operations = [
        migrations.CreateModel(
            name='GoodbyeMessage',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.TextField(verbose_name='Goodbye Message')),
                ('authenticated', models.BooleanField(
                    verbose_name='Valid for Authenticated Users')),
                ('unauthenticated', models.BooleanField(
                    verbose_name='Valid for Un-Authenticated Users')),
            ],
            options={
                'verbose_name': 'Goodbye Message',
                'verbose_name_plural': 'Goodbye Messages',
                'default_permissions': (),
            },
        ),
        migrations.CreateModel(
            name='WelcomeMessage',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.TextField(verbose_name='Welcome Message')),
                ('authenticated', models.BooleanField(
                    verbose_name='Valid for Authenticated Users')),
                ('unauthenticated', models.BooleanField(
                    verbose_name='Valid for Un-Authenticated Users')),
            ],
            options={
                'verbose_name': 'Welcome Message',
                'verbose_name_plural': 'Welcome Messages',
                'default_permissions': (),
            },
        ),
    ]
